;;; polish.el 1.2.2 --- Make ISO-Latin-2 Polish characters accessible from the keyboard.
;;; by J. Chroboczek -- jec@dcs.ed.ac.uk

;;; to use, add the following lines to your .emacs:

;;; (standard-display-european t)
;;; (autoload 'polish-mode "polish" "Mode for typing ogonki" t)
;;; (autoload 'latin2-font "polish" "Switch fonts" t)

;;; polish-mode is a minor mode for typing ogonki; type eg. /a for
;;; a-ogonek, /l for l-bar, and /x for ziet.  It is buffer local.

;;; latin2-font switches the current frame to/from an ISO-8859-2 font.

;;; In order to use latin2-font, polish-latin1-font should be the font
;;; you normally use; polish-latin2-font should be some ISO-8859-2
;;; encoded font.  If you want font-lock, bold and italic versions of
;;; polish-latin2-font should be installed.
;;; Of course, if you use an ISO-8859-2 font by default, you do not
;;; need latin2-font.

;;; For customization, see variables polish-prefix-char and polish-keys.

;;; LIMITATIONS: 
;;;  o this code should not be loaded from .emacs under a windowing
;;;    system; either autoload it or load it from a hook;
;;;  o latin2-font doesn't work in XEmacs; it would be easy to do, but
;;;    I'm not interested.  Diffs welcome!

(provide 'polish)

(defvar polish-latin1-font
  (or (cdr (assoc 'font (frame-parameters)))
      "-adobe-courier-medium-r-normal--12-120-75-75-m-70-iso8859-1")
  "The font for Latin-1")
(defvar polish-latin2-font 
  "-adobe-courier-medium-r-*--14-*-*-*-m-*-iso8859-2"
  "The font for Latin-2")

(defvar polish-prefix-char ?/
  "The character used for prefixing ogonki in polish-mode")

(defvar polish-keys 
  `((?e . 234) (?E . 202) (?o . 243) (?O . 211) (?a . 177) (?A . 161)
    (?s . 182) (?S . 166) (?l . 179) (?L . 163) (?x . 188) (?X . 172)
    (?z . 191) (?Z . 175) (?c . 230) (?C . 198) (?n . 241) (?N . 209)
    (,polish-prefix-char . ,polish-prefix-char))
  "Association list containing the prefixed polish mode key bindings")

(defun polish-insert-command (arg)
  "Insert a Polish character."
  (interactive "p")
  (let ((pair (assoc last-command-char polish-keys)))
    (if pair
        (insert-char (cdr pair) arg)
      (error ""))))

(defvar polish-keymap (make-sparse-keymap) 
  "Keymap for polish-mode")
(defvar polish-secondary-keymap (make-sparse-keymap) 
  "Secondary keymap for polish-mode")

;;; actually define the Polish characters
(define-key polish-keymap 
  (char-to-string polish-prefix-char) polish-secondary-keymap)
(mapcar 
 (function
  (lambda (keys-entry)
    (define-key polish-secondary-keymap
      (char-to-string (car keys-entry))
      'polish-insert-command)))
 polish-keys)

(or (assq 'polish-mode minor-mode-map-alist)
    (setq minor-mode-map-alist (cons (cons 'polish-mode polish-keymap) 
				     minor-mode-map-alist)))

(or (assq 'polish-mode minor-mode-alist)
    (setq minor-mode-alist (cons (list 'polish-mode " Polish") 
				 minor-mode-alist)))
(defun polish-mode ()
  "Mode for accessing Polish characters from the keyboard.
A polish letter is accessed by typing `/' followed by the closest letter
in the Latin alphabet."
  (interactive)
  (if (boundp 'polish-mode)
      nil
    (make-variable-buffer-local 'polish-mode)
    (set-default 'polish-mode nil))
  (setq polish-mode (not polish-mode)))

(defvar polish-latin2-frames '()
  "The set of frames currently using a Latin-2 font")

(defun latin2-font (&optional frame)
  "Switch the frame between latin1 and latin2 fonts"
  (interactive)
  (if frame
      nil
    (setq frame (selected-frame)))
  (cond
   ((member frame polish-latin2-frames)
    (modify-frame-parameters frame (list (cons 'font polish-latin1-font)))
    (setq polish-latin2-frames (delete frame polish-latin2-frames)))
   (t
    (modify-frame-parameters frame (list (cons 'font polish-latin2-font)))
    (setq polish-latin2-frames (cons frame polish-latin2-frames)))))
